#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "myutil.h"

void muEC(int position) //checks and outputs error position and error string
{
	cudaError_t errcode = cudaGetLastError();
	if(errcode==cudaSuccess)
	{
		printf("No error at position %i\n", position);
		return;
	}
	printf("Error position: %i\nCode:%s\n", position, cudaGetErrorString(errcode));
}
void muRC(int position, CUresult result)
{
	if(result==0)
		printf("Success at %i\n", position);
	else
		printf("Error at %i:%i\n", position, result);
} //Error codes attached at the end of the file

int muV(float* target1, float* target2, int length) //Verification. Outputs position of first difference if there is any
{
	for(int i=0; i<length; i++)
	{
		if(target1[i]!=target2[i])
		{
			printf("Verification failed at i=%i\n", i);
			return i;
		}
	}
	puts("Verification passed");
	return -1;
}
__global__ void mu__W(int cycles) //Time killer, does 2*cycles of calculation... to get timer running?
{
	int i = 0;
	for(;i<cycles; i++)
	{
		i++;i--;
	}
}
void muW(int cycles) //wrapper of mu__W, time killer.
{
	dim3 gridSize(7,1,1);
	dim3 blockSize(32,1,1);
	mu__W<<<gridSize, blockSize>>>(cycles);
}
int muDriverVersion() //prints driver version
{
	int result;
	cudaDriverGetVersion(&result);
	printf("Driver Version: %i\n", result);
	return result;
}
int muRuntimeVersion() //prints runtime version
{
	int result;
	cudaRuntimeGetVersion(&result);
	printf("Runtime Version: %i\n", result);
	return result;
}


/*
 ************************************
 **
 **    Error codes
 **
 ***********************************

typedef enum cudaError_enum {

    CUDA_SUCCESS                    = 0,
    CUDA_ERROR_INVALID_VALUE        = 1,
    CUDA_ERROR_OUT_OF_MEMORY        = 2,
    CUDA_ERROR_NOT_INITIALIZED      = 3,
    CUDA_ERROR_DEINITIALIZED        = 4,

    CUDA_ERROR_NO_DEVICE            = 100,
    CUDA_ERROR_INVALID_DEVICE       = 101,

    CUDA_ERROR_INVALID_IMAGE        = 200,
    CUDA_ERROR_INVALID_CONTEXT      = 201,
    CUDA_ERROR_CONTEXT_ALREADY_CURRENT = 202,
    CUDA_ERROR_MAP_FAILED           = 205,
    CUDA_ERROR_UNMAP_FAILED         = 206,
    CUDA_ERROR_ARRAY_IS_MAPPED      = 207,
    CUDA_ERROR_ALREADY_MAPPED       = 208,
    CUDA_ERROR_NO_BINARY_FOR_GPU    = 209,
    CUDA_ERROR_ALREADY_ACQUIRED     = 210,
    CUDA_ERROR_NOT_MAPPED           = 211,

    CUDA_ERROR_INVALID_SOURCE       = 300,
    CUDA_ERROR_FILE_NOT_FOUND       = 301,

    CUDA_ERROR_INVALID_HANDLE       = 400,

    CUDA_ERROR_NOT_FOUND            = 500,

    CUDA_ERROR_NOT_READY            = 600,

    CUDA_ERROR_LAUNCH_FAILED        = 700,
    CUDA_ERROR_LAUNCH_OUT_OF_RESOURCES = 701,
    CUDA_ERROR_LAUNCH_TIMEOUT       = 702,
    CUDA_ERROR_LAUNCH_INCOMPATIBLE_TEXTURING = 703,

    CUDA_ERROR_UNKNOWN              = 999
} CUresult;
*/